#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <GL/glew.h>
#ifdef __APPLE__
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif
#include "app.h"

static bool init();
static void display();
static void idle();
static void reshape(int x, int y);
static void key_press(unsigned char key, int x, int y);
static void key_release(unsigned char key, int x, int y);
static void mouse(int bn, int st, int x, int y);

static unsigned int start_time;

int main(int argc, char **argv)
{
	glutInitWindowSize(1024, 768);
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_RGB | GLUT_DEPTH | GLUT_DOUBLE | GLUT_MULTISAMPLE);
	glutCreateWindow("demo");

	glutDisplayFunc(display);
	glutIdleFunc(idle);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(key_press);
	glutKeyboardUpFunc(key_release);
	glutMouseFunc(mouse);
	glutMotionFunc(app_mouse_motion);
	glutPassiveMotionFunc(app_mouse_motion);

	if(!init()) {
		return 1;
	}
	atexit(app_cleanup);

	glutMainLoop();
	return 0;
}

void app_swap_buffers()
{
	glutSwapBuffers();
}

void app_quit()
{
	exit(0);
}

static bool init()
{
	glewInit();

	if(!app_init()) {
		return false;
	}

	start_time = glutGet(GLUT_ELAPSED_TIME);
	return true;
}

static void display()
{
	time_msec = glutGet(GLUT_ELAPSED_TIME) - start_time;
	app_display();
}

static void idle()
{
	glutPostRedisplay();
}

static void reshape(int x, int y)
{
	win_width = x;
	win_height = y;

	app_reshape(x, y);
}

static void key_press(unsigned char key, int x, int y)
{
	app_keyboard(key, true);
}

static void key_release(unsigned char key, int x, int y)
{
	app_keyboard(key, false);
}

static void mouse(int bn, int st, int x, int y)
{
	int bidx = bn - GLUT_LEFT_BUTTON;
	bool down = st == GLUT_DOWN;

	app_mouse_button(bidx, down, x, y);
}
